
#include <QQmlEngine>
#include <QtQml/private/qqmlmetatype_p.h>

#include <QTextStream>
#include <QGuiApplication>
#include <QTimer>

#include "rpmbqml-qmlinfo-helper.h"

struct Versions {
	int minMinorVersion;
	int maxMinorVersion;
	int majorVersion;
};


Helper::Helper(QObject *parent, const QString& arg1, const QString& arg2):
    QObject(parent),
    m_arg1(arg1),
    m_arg2(arg2)
{
	init();
}

Helper::~Helper()
{
}

void Helper::init()
{
	QQmlEngine engine(this);

	auto start_types = QQmlMetaType::qmlAllTypes();

	QList<QQmlError> errors;
	QHash<QString, Versions> types;
	bool res = engine.importPlugin(m_arg1, m_arg2, &errors);

	if (!res)
		qWarning("Import failed");

	foreach (const QQmlError &error, errors)
		qWarning("Error: %s", error.toString().toLocal8Bit().data());

	auto end_types = QQmlMetaType::qmlAllTypes();

	foreach (const QQmlType ty, end_types)
	{
		bool found = false;

		for (auto it = start_types.begin(); it != start_types.end(); ++it)
		{
			if ((*it).typeName() == ty.typeName())
			{
				found = true;
				break;
			}
		}

		if (!found && ty.createSize() > 0 && ty.qmlTypeName().toLocal8Bit().length() > 0)
		{
			QString str = ty.qmlTypeName().toLocal8Bit().data();
			if (types.contains(str)) {
				for (QHash<QString, Versions>::iterator it = types.find(str);
					it != types.end(); ++it)
				{
					if (it.key() == str) {
						(*it).majorVersion = ty.majorVersion();
						(*it).maxMinorVersion = qMax((*it).maxMinorVersion, ty.minorVersion());
						(*it).minMinorVersion = qMin((*it).minMinorVersion, ty.minorVersion());
					}
				}
			}
			else
			{
				Versions ver = {
					ty.minorVersion(),
					ty.minorVersion(),
					ty.majorVersion()
				};
				types.insert(str, ver);
			}
		}
	}

	for (QHash<QString, Versions>::const_iterator it = types.begin(); it != types.end(); ++it)
	{
		QTextStream out(stdout);
		for (int i = (*it).minMinorVersion; i<=(*it).maxMinorVersion; i++) {
			out << it.key().toLocal8Bit().data() << " " << (*it).majorVersion << "." << i << endl;
		}
	}

	//QGuiApplication::quit();
}
